@@ -5,6 +5,7 @@
 
 
 
+
 /********************************************************
 
    LIP INTERFACE 
@@ -29,13 +30,11 @@ NTL_verylong rep; // This is currently public for "emergency" situations
                    // May be private in future versions.
 
 
-ZZ() 
+ZZ() : rep(0) { }
 // initial value is 0.
 
-{ rep = 0; }
 
-
-ZZ(INIT_SIZE_TYPE, long k)
+ZZ(INIT_SIZE_TYPE, long k) : rep(0)
 // initial value is 0, but space is pre-allocated so that numbers
 // x with x.size() <= k can be stored without re-allocation.
 // Call with ZZ(INIT_SIZE, k).
@@ -44,24 +43,22 @@ ZZ(INIT_SIZE_TYPE, long k)
 
 
 {
-   rep = 0;
    NTL_zsetlength(&rep, k); 
 }
 
-ZZ(const ZZ& a)
+ZZ(const ZZ& a) : rep(0)
 // initial value is a.
 
 {
-   rep = 0;
    NTL_zcopy(a.rep, &rep);
 }
 
 
-ZZ(INIT_VAL_TYPE, long a) { rep = 0; NTL_zintoz(a, &rep); }
-ZZ(INIT_VAL_TYPE, int a) { rep = 0; NTL_zintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, long a) : rep(0) { NTL_zintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, int a) : rep(0) { NTL_zintoz(a, &rep); }
 
-ZZ(INIT_VAL_TYPE, unsigned long a) { rep = 0; NTL_zuintoz(a, &rep); }
-ZZ(INIT_VAL_TYPE, unsigned int a) { rep = 0; NTL_zuintoz((unsigned long) a, &rep); }
+ZZ(INIT_VAL_TYPE, unsigned long a) : rep(0) { NTL_zuintoz(a, &rep); }
+ZZ(INIT_VAL_TYPE, unsigned int a) : rep(0) { NTL_zuintoz((unsigned long) a, &rep); }
 
 inline ZZ(INIT_VAL_TYPE, const char *);
 inline ZZ(INIT_VAL_TYPE, float);
@@ -88,9 +85,16 @@ void SetSize(long k)
 { NTL_zsetlength(&rep, k); }
 
 long size() const
+// returns the number of (NTL_ZZ_NBIT-bit) digits of |a|; the size of 0 is 0.
    { return NTL_zsize(rep); }
 
-// returns the number of (NTL_ZZ_NBIT-bit) digits of |a|; the size of 0 is 0.
+long null() const
+// test of rep is null
+   { return !rep; }
+
+long MaxAlloc() const
+// returns max allocation request, possibly rounded up a bit...
+   { return NTL_zmaxalloc(rep); }
 
 
 long SinglePrecision() const
@@ -165,15 +169,15 @@ inline void conv(ZZ& x, unsigned int a) { NTL_zuintoz((unsigned long)(a), &x.rep
 inline ZZ to_ZZ(unsigned int a) { return ZZ(INIT_VAL, a); }
 
 void conv(ZZ& x, const char *s);
-inline ZZ::ZZ(INIT_VAL_TYPE, const char *s) {  rep = 0; conv(*this, s); }
+inline ZZ::ZZ(INIT_VAL_TYPE, const char *s) : rep(0) { conv(*this, s); }
 inline ZZ to_ZZ(const char *s) { return ZZ(INIT_VAL, s); }
 
 inline void conv(ZZ& x, double a) { NTL_zdoubtoz(a, &x.rep); }
-inline ZZ::ZZ(INIT_VAL_TYPE, double a) { rep = 0; conv(*this, a); }
+inline ZZ::ZZ(INIT_VAL_TYPE, double a) : rep(0) { conv(*this, a); }
 inline ZZ to_ZZ(double a) { return ZZ(INIT_VAL, a); }
 
 inline void conv(ZZ& x, float a) { NTL_zdoubtoz(double(a), &x.rep); }
-inline ZZ::ZZ(INIT_VAL_TYPE, float a) { rep = 0; conv(*this, a); }
+inline ZZ::ZZ(INIT_VAL_TYPE, float a) : rep(0) { conv(*this, a); }
 inline ZZ to_ZZ(float a) { return ZZ(INIT_VAL, a); }
 
 inline void conv(long& x, const ZZ& a) { x = NTL_ztoint(a.rep); }
@@ -419,6 +423,34 @@ inline ZZ& operator*=(ZZ& x, const ZZ& a)
 inline ZZ& operator*=(ZZ& x, long a)
   { mul(x, x, a); return x; }
 
+// x += a*b
+
+inline void
+MulAddTo(ZZ& x, const ZZ& a, long b)
+{
+   NTL_zsaddmul(a.rep, b, &x.rep);
+}
+
+inline void
+MulAddTo(ZZ& x, const ZZ& a, const ZZ& b)
+{
+   NTL_zaddmul(a.rep, b.rep, &x.rep);
+}
+
+// x -= a*b
+
+inline void
+MulSubFrom(ZZ& x, const ZZ& a, long b)
+{
+   NTL_zssubmul(a.rep, b, &x.rep);
+}
+
+inline void
+MulSubFrom(ZZ& x, const ZZ& a, const ZZ& b)
+{
+   NTL_zsubmul(a.rep, b.rep, &x.rep);
+}
+
 
 // Special routines for implementing CRT in ZZ_pX arithmetic
 
@@ -1015,6 +1047,7 @@ inline long SqrRoot(long a) { return NTL_zsqrts(a); }
 // All args (other than exponents) are assumed to be in the range 0..n-1.
 
 
+
 inline void AddMod(ZZ& x, const ZZ& a, const ZZ& b, const ZZ& n)
 // x = (a+b)%n
    { NTL_zaddmod(a.rep, b.rep, n.rep, &x.rep); }
@@ -1223,9 +1256,21 @@ If n is the modulus, all inputs should be in the range 0..n-1.
 The number n itself should be in the range 1..2^{NTL_SP_NBITS}-1.
 */
 
+// I've declared these "static" so that the installation wizard
+// has more flexibility, without worrying about the (rather esoteric)
+// possibility of the linker complaining when the definitions
+// are inconsistent across severeal files.
+// Maybe an unnamed namespace would be better.
 
 
-inline long AddMod(long a, long b, long n)
+
+
+
+
+
+
+
+static inline long AddMod(long a, long b, long n)
 // return (a+b)%n
 
 {
@@ -1234,6 +1279,10 @@ inline long AddMod(long a, long b, long n)
    res -= n;
    res += (res >> (NTL_BITS_PER_LONG-1)) & n;
    return res;
+#elif (defined(NTL_AVOID_BRANCHING))
+   res -= n;
+   res += (long) ((-(((unsigned long) res) >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n));
+   return res;
 #else
    if (res >= n)
       return res - n;
@@ -1242,7 +1291,7 @@ inline long AddMod(long a, long b, long n)
 #endif
 }
 
-inline long SubMod(long a, long b, long n)
+static inline long SubMod(long a, long b, long n)
 // return (a-b)%n
 
 {
@@ -1250,6 +1299,9 @@ inline long SubMod(long a, long b, long n)
 #if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING) && !defined(NTL_CLEAN_INT))
    res += (res >> (NTL_BITS_PER_LONG-1)) & n;
    return res;
+#elif (defined(NTL_AVOID_BRANCHING))
+   res += (long) ((-(((unsigned long) res) >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n));
+   return res;
 #else
    if (res < 0)
       return res + n;
@@ -1258,19 +1310,25 @@ inline long SubMod(long a, long b, long n)
 #endif
 }
 
-inline long NegateMod(long a, long n)
+static inline long NegateMod(long a, long n)
 {
    return SubMod(0, a, n);
 }
 
 
+
+#if (defined(NTL_CLEAN_INT) || (defined(NTL_AVOID_BRANCHING)  && !NTL_ARITH_RIGHT_SHIFT))
+#define NTL_CLEAN_SPMM
+#endif
+
+
 #if (defined(NTL_SINGLE_MUL))
 
 
 #if (!defined(NTL_FAST_INT_MUL))
 
 
-inline long MulMod(long a, long b, long n)
+static inline long MulMod(long a, long b, long n)
 // return (a*b)%n
 
 {
@@ -1299,7 +1357,7 @@ which is assumed to equal 1/((double) n).
 It is usually faster than the above.
 */
 
-inline long MulMod(long a, long b, long n, double ninv)
+static inline long MulMod(long a, long b, long n, double ninv)
 {
    double ab;
    long q, res;
@@ -1325,7 +1383,7 @@ Yet another MulMod.
 This time, the 4th argument should be ((double) b)/((double) n).
 */
 
-inline long MulMod2(long a, long b, long n, double bninv)
+static inline long MulMod2(long a, long b, long n, double bninv)
 {
    double ab;
    long q, res;
@@ -1346,7 +1404,7 @@ inline long MulMod2(long a, long b, long n, double bninv)
    return res;
 }
 
-inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
+static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 {
    double ab;
    long q, res;
@@ -1368,7 +1426,7 @@ inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 
 #else
 
-inline long MulMod(long a, long b, long n)
+static inline long MulMod(long a, long b, long n)
 // return (a*b)%n
 
 {
@@ -1402,7 +1460,7 @@ which is assumed to equal 1/((double) n).
 It is usually faster than the above.
 */
 
-inline long MulMod(long a, long b, long n, double ninv)
+static inline long MulMod(long a, long b, long n, double ninv)
 {
    double ab, xx;
    long iab, q, res;
@@ -1432,7 +1490,7 @@ Yet another MulMod.
 This time, the 4th argument should be ((double) b)/((double) n).
 */
 
-inline long MulMod2(long a, long b, long n, double bninv)
+static inline long MulMod2(long a, long b, long n, double bninv)
 {
    long q, res;
 
@@ -1452,7 +1510,7 @@ inline long MulMod2(long a, long b, long n, double bninv)
 }
 
 
-inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
+static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 {
    long q, res;
 
@@ -1473,9 +1531,14 @@ inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 #endif
 
 
-#elif (!defined(NTL_CLEAN_INT))
+#elif (!defined(NTL_CLEAN_SPMM))
 
-inline long MulMod(long a, long b, long n)
+
+/*
+ * The default MulMod code.
+ */
+
+static inline long MulMod(long a, long b, long n)
 {
    long q, res;
 
@@ -1494,7 +1557,7 @@ inline long MulMod(long a, long b, long n)
    return res;
 }
 
-inline long MulMod(long a, long b, long n, double ninv)
+static inline long MulMod(long a, long b, long n, double ninv)
 {
    long q, res;
 
@@ -1514,7 +1577,7 @@ inline long MulMod(long a, long b, long n, double ninv)
 }
 
 
-inline long MulMod2(long a, long b, long n, double bninv)
+static inline long MulMod2(long a, long b, long n, double bninv)
 {
    long q, res;
 
@@ -1533,7 +1596,7 @@ inline long MulMod2(long a, long b, long n, double bninv)
    return res;
 }
 
-inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
+static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 {
    long q, res;
 
@@ -1558,8 +1621,8 @@ inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
  * assuming IEEE floating point arithmetic.
  */
 
-inline long MulMod(long a, long b, long n)
-{
+static inline long MulMod(long a, long b, long n)
+{  
    long q;
    unsigned long res;
 
@@ -1568,15 +1631,21 @@ inline long MulMod(long a, long b, long n)
    res = ((unsigned long) a)*((unsigned long) b) - 
          ((unsigned long) q)*((unsigned long) n);
 
+#if (defined(NTL_AVOID_BRANCHING))
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+   res -= ((unsigned long) n);
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+#else
    if (res >> (NTL_BITS_PER_LONG-1))
-      res += n;
+      res += ((unsigned long) n);
    else if (((long) res) >= n)
-      res -= n;
+      res -= ((unsigned long) n);
+#endif
  
    return ((long) res);
 }
 
-inline long MulMod(long a, long b, long n, double ninv)
+static inline long MulMod(long a, long b, long n, double ninv)
 {
    long q; 
    unsigned long res;
@@ -1586,16 +1655,22 @@ inline long MulMod(long a, long b, long n, double ninv)
    res = ((unsigned long) a)*((unsigned long) b) - 
          ((unsigned long) q)*((unsigned long) n);
 
+#if (defined(NTL_AVOID_BRANCHING))
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+   res -= ((unsigned long) n);
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+#else
    if (res >> (NTL_BITS_PER_LONG-1))
-      res += n;
+      res += ((unsigned long) n);
    else if (((long) res) >= n)
-      res -= n;
+      res -= ((unsigned long) n);
+#endif
  
    return ((long) res);
 }
 
 
-inline long MulMod2(long a, long b, long n, double bninv)
+static inline long MulMod2(long a, long b, long n, double bninv)
 {
    long q;
    unsigned long res;
@@ -1605,15 +1680,22 @@ inline long MulMod2(long a, long b, long n, double bninv)
    res = ((unsigned long) a)*((unsigned long) b) - 
          ((unsigned long) q)*((unsigned long) n);
 
+#if (defined(NTL_AVOID_BRANCHING))
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+   res -= ((unsigned long) n);
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+#else
    if (res >> (NTL_BITS_PER_LONG-1))
-      res += n;
+      res += ((unsigned long) n);
    else if (((long) res) >= n)
-      res -= n;
+      res -= ((unsigned long) n);
+#endif
+
  
    return ((long) res);
 }
 
-inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
+static inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 {
    long q; 
    unsigned long res;
@@ -1638,6 +1720,297 @@ inline long MulDivRem(long& qq, long a, long b, long n, double bninv)
 #endif
 
 
+
+
+// These MulMod routines (with preconditioning) are sometimes
+// significantly faster.  There are four possible implementations:
+//  - default: uses MulMod2 above (lots of floating point)
+//  - NTL_SPMM_ULL: uses unsigned long long (if possible)
+//  - NTL_SPMM_ASM: uses assembly language (if possible)
+//  - NTL_SPMM_UL: uses only unsigned long arithmetic (portable, slower).
+
+#if (!defined(NTL_SINGLE_MUL) && (defined(NTL_SPMM_ULL) || defined(NTL_SPMM_ASM)))
+
+
+// unsigned long long / asm versions
+
+typedef unsigned long mulmod_precon_t;
+
+#define NTL_SPMM_VEC_T vec_ulong
+
+#if (!defined(NTL_CLEAN_SPMM))
+
+static inline unsigned long PrepMulModPrecon(long b, long n, double ninv)
+{
+   long q, r;
+
+   q  = (long) ( (((double) b) * NTL_SP_FBOUND) * ninv ); 
+   r = (b << NTL_SP_NBITS) - q*n;
+
+#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
+   q += 1 + (r >> (NTL_BITS_PER_LONG-1)) + ((r - n) >> (NTL_BITS_PER_LONG-1));
+#else
+   if (r >= n)
+      q++;
+   else if (r < 0)
+      q--;
+#endif
+
+   return ((unsigned long) q) << (NTL_BITS_PER_LONG - NTL_SP_NBITS);
+}
+
+
+#else
+
+/*
+ * clean int version -- this should be completely portable.
+ */
+
+
+static inline unsigned long PrepMulModPrecon(long b, long n, double ninv)
+{
+   unsigned long q, r;
+
+   q = (long) ( (((double) b) * NTL_SP_FBOUND) * ninv ); 
+   r = (((unsigned long) b) << NTL_SP_NBITS ) - q * ((unsigned long) n);
+
+#if (defined(NTL_AVOID_BRANCHING))
+   q += 1UL - (r >> (NTL_BITS_PER_LONG-1)) - ((r - ((unsigned long) n)) >> (NTL_BITS_PER_LONG-1));
+#else
+   if (r >> (NTL_BITS_PER_LONG-1))
+      q--;
+   else if (((long) r) >= n)
+      q++;
+#endif
+
+   return q << (NTL_BITS_PER_LONG - NTL_SP_NBITS);
+}
+
+
+
+#endif
+
+
+
+
+#if (defined(NTL_SPMM_ULL))
+
+static inline unsigned long MulHiUL(unsigned long a, unsigned long b)
+{
+   return (((NTL_ULL_TYPE)(a)) * ((NTL_ULL_TYPE)(b))) >> NTL_BITS_PER_LONG;
+} 
+
+#else 
+
+// assmbly code versions
+
+#include <NTL/SPMM_ASM.h>
+
+
+#endif
+
+
+
+
+
+   
+
+#if (!defined(NTL_CLEAN_SPMM))
+
+
+
+static inline long MulModPrecon(long a, long b, long n, unsigned long bninv)
+{
+   long q, res;
+   
+   q = (long) MulHiUL(a, bninv);
+
+   res = a*b - q*n;
+#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
+   res -= n;
+   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
+#else
+   if (res >= n)
+      res -= n;
+#endif
+   return res;
+}
+
+
+#else
+
+static inline long MulModPrecon(long a, long b, long n, unsigned long bninv)
+{
+   unsigned long q, res;
+
+   
+   q = MulHiUL(a, bninv);
+
+   res = ((unsigned long) a)*((unsigned long) b) - q*((unsigned long) n);
+
+#if (defined(NTL_AVOID_BRANCHING))
+   res -= ((unsigned long) n);
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+#else
+   if (((long) res) >= n)
+      res -= ((unsigned long) n);
+#endif
+
+   return (long) res;
+}
+
+#endif
+
+
+
+#elif (!defined(NTL_SINGLE_MUL) && defined(NTL_SPMM_UL))
+
+// plain, portable (but slower) int version
+
+typedef long mulmod_precon_t;
+
+#define NTL_SPMM_VEC_T vec_long
+
+
+#if (!defined(NTL_CLEAN_SPMM))
+
+static inline long PrepMulModPrecon(long b, long n, double ninv)
+{
+   long q, r;
+
+   q  = (long) ( (((double) b) * NTL_SP_FBOUND) * ninv ); 
+   r = (b << NTL_SP_NBITS) - q*n;
+
+
+#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
+   q += 1 + (r >> (NTL_BITS_PER_LONG-1)) + ((r - n) >> (NTL_BITS_PER_LONG-1));
+#else
+   if (r >= n)
+      q++;
+   else if (r < 0)
+      q--;
+#endif
+
+   return q;
+}
+
+
+#else
+
+static inline long PrepMulModPrecon(long b, long n, double ninv)
+{
+   unsigned long q, r;
+
+   q = (long) ( (((double) b) * NTL_SP_FBOUND) * ninv ); 
+   r = (((unsigned long) b) << NTL_SP_NBITS ) - q * ((unsigned long) n);
+
+#if (defined(NTL_AVOID_BRANCHING))
+   q += 1UL - (r >> (NTL_BITS_PER_LONG-1)) - ((r - ((unsigned long) n)) >> (NTL_BITS_PER_LONG-1));
+#else
+   if (r >> (NTL_BITS_PER_LONG-1))
+      q--;
+   else if (((long) r) >= n)
+      q++;
+#endif
+
+   return ((long) q);
+}
+
+
+#endif
+
+
+
+
+static inline long MulHiSP(long b, long d)
+{
+   unsigned long _b1 = b & ((1UL << (NTL_SP_NBITS/2)) - 1UL);
+   unsigned long _d1 = d & ((1UL << (NTL_SP_NBITS/2)) - 1UL);
+   unsigned long _bd,_b1d1,_m,_aa;
+   unsigned long _ld = (d>>(NTL_SP_NBITS/2));
+   unsigned long _lb = (b>>(NTL_SP_NBITS/2));
+
+   _bd=_lb*_ld;
+   _b1d1=_b1*_d1;
+   _m=(_lb+_b1)*(_ld+_d1) - _bd - _b1d1;
+   _aa = ( _b1d1+ ((_m&((1UL << (NTL_SP_NBITS/2)) - 1UL))<<(NTL_SP_NBITS/2)));
+   return (_aa >> NTL_SP_NBITS) + _bd + (_m>>(NTL_SP_NBITS/2));
+}
+
+
+#if (!defined(NTL_CLEAN_SPMM))
+
+static inline long MulModPrecon(long a, long b, long n, long bninv)
+{
+
+   long q, res;
+
+   q = MulHiSP(a, bninv);
+
+   res = a*b - q*n;
+#if (NTL_ARITH_RIGHT_SHIFT && defined(NTL_AVOID_BRANCHING))
+   res -= n;
+   res += (res >> (NTL_BITS_PER_LONG-1)) & n;
+#else
+   if (res >= n)
+      res -= n;
+#endif
+   return res;
+}
+
+
+
+#else
+
+
+static inline long MulModPrecon(long a, long b, long n, long bninv)
+{
+
+   unsigned long q, res;
+
+   q = MulHiSP(a, bninv);
+
+   res = ((unsigned long) a)*((unsigned long) b) - q*((unsigned long) n);
+
+#if (defined(NTL_AVOID_BRANCHING))
+   res -= ((unsigned long) n);
+   res += (-(res >> (NTL_BITS_PER_LONG-1))) & ((unsigned long) n);
+#else
+   if (((long) res) >= n)
+      res -= ((unsigned long) n);
+#endif
+
+   return (long) res;
+}
+
+
+#endif
+
+
+
+
+#else
+
+// default, double version
+
+typedef double mulmod_precon_t;
+
+#define NTL_SPMM_VEC_T vec_double
+
+static inline double PrepMulModPrecon(long b, long n, double ninv)
+{
+   return ((double) b) * ninv;
+}
+
+static inline long MulModPrecon(long a, long b, long n, double bninv)
+{
+   return MulMod2(a, b, n, bninv);
+}
+
+
+#endif
+
+
 long InvMod(long a, long n);
 // computes a^{-1} mod n.  Error is raised if undefined.
 
@@ -1645,6 +2018,32 @@ long PowerMod(long a, long e, long n);
 // computes a^e mod n, e >= 0
 
 
+inline
+void VectorMulModPrecon(long k, long *x, const long *a, long b, long n, 
+                        mulmod_precon_t bninv)
+{
+   for (long i = 0; i < k; i++)
+      x[i] = MulModPrecon(a[i], b, n, bninv);
+}
+
+inline
+void VectorMulMod(long k, long *x, const long *a, long b, long n, 
+                  double ninv)
+{
+   mulmod_precon_t bninv;
+   bninv = PrepMulModPrecon(b, n, ninv);
+   VectorMulModPrecon(k, x, a, b, n, bninv);
+}
+
+
+inline 
+void VectorMulMod(long k, long *x, const long *a, long b, long n)
+{
+   double ninv = 1/((double) n);
+   VectorMulMod(k, x, a, b, n, ninv);
+}
+
+
 NTL_CLOSE_NNS
 
 
