@@ -193,6 +193,19 @@ extern "C" {
 	  The division is performed in place (but may sometimes
           cause *r to grow by one digit) */
 
+    void _ntl_zsaddmul(_ntl_verylong x, long y,  _ntl_verylong *ww);
+      /* *ww += x*y */
+
+    void _ntl_zaddmul(_ntl_verylong x, _ntl_verylong y,  _ntl_verylong *ww);
+      /* *ww += x*y */
+
+    void _ntl_zssubmul(_ntl_verylong x, long y,  _ntl_verylong *ww);
+      /* *ww -= x*y */
+
+    void _ntl_zsubmul(_ntl_verylong x, _ntl_verylong y,  _ntl_verylong *ww);
+      /* *ww -= x*y */
+
+
 /********************************************************************
 
    Shifting and bit manipulation
@@ -498,6 +511,10 @@ extern "C" {
 ***********************************************************************/
 
 
+    long _ntl_zmaxalloc(_ntl_verylong x); 
+      /* max allocation request, possibly rounded up a bit */
+
+
     void _ntl_zsetlength(_ntl_verylong *v, long len);
        /* Allocates enough space to hold a len-digit number,
           where each digit has NTL_NBITS bits.
@@ -614,6 +631,7 @@ extern int _ntl_gmp_hack;
 #define NTL_zscompare _ntl_zscompare
 #define NTL_zsdiv _ntl_zsdiv
 #define NTL_zsetbit _ntl_zsetbit
+#define NTL_zmaxalloc _ntl_zmaxalloc
 #define NTL_zsetlength _ntl_zsetlength
 #define NTL_zsign _ntl_zsign
 #define NTL_zsize _ntl_zsize
@@ -641,3 +659,8 @@ extern int _ntl_gmp_hack;
 #define NTL_zxxratrecon _ntl_zxxratrecon
 #define NTL_zzero _ntl_zzero1
 
+#define NTL_zsaddmul _ntl_zsaddmul
+#define NTL_zaddmul _ntl_zaddmul
+#define NTL_zssubmul _ntl_zssubmul
+#define NTL_zsubmul _ntl_zsubmul
+
